@@ -1,22 +1,18 @@
-# Module:   debugger
-# Date:     2nd April 2006
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """
 Debugger component used to debug each event in a system by printing
 each event to sys.stderr or to a Logger Component instance.
 """
-
 import os
 import sys
-from traceback import format_exc
-
+from signal import SIGINT, SIGTERM
+from traceback import format_exc, format_exception_only
 
 from .components import BaseComponent
 from .handlers import handler, reprhandler
 
 
 class Debugger(BaseComponent):
+
     """Create a new Debugger Component
 
     Creates a new Debugger Component that listens to all events in the system
@@ -38,8 +34,8 @@ class Debugger(BaseComponent):
 
         super(Debugger, self).__init__()
 
-        self.errors = errors
-        self.events = events
+        self._errors = errors
+        self._events = events
 
         if isinstance(file, str):
             self.file = open(os.path.abspath(os.path.expanduser(file)), "a")
@@ -55,11 +51,16 @@ class Debugger(BaseComponent):
         self.IgnoreEvents.extend(kwargs.get("IgnoreEvents", []))
         self.IgnoreChannels.extend(kwargs.get("IgnoreChannels", []))
 
-    @handler("error", channel="*", priority=100.0)
-    def _on_error(self, error_type, value, traceback,
-                  handler=None, fevent=None):
+    @handler("signal", channel="*")
+    def _on_signal(self, signo, stack):
+        if signo in [SIGINT, SIGTERM]:
+            raise SystemExit(0)
 
-        if not self.errors:
+    @handler("exception", channel="*", priority=100.0)
+    def _on_exception(self, error_type, value, traceback,
+                      handler=None, fevent=None):
+
+        if not self._errors:
             return
 
         s = []
@@ -69,9 +70,14 @@ class Debugger(BaseComponent):
         else:
             handler = reprhandler(handler)
 
-        msg = "ERROR %s (%s) {%s}: %s\n" % (handler, fevent, error_type, value)
+        msg = "ERROR {0:s} ({1:s}) ({2:s}): {3:s}\n".format(
+            handler, repr(fevent), repr(error_type), repr(value)
+        )
+
         s.append(msg)
+        s.append('Traceback (most recent call last):\n')
         s.extend(traceback)
+        s.extend(format_exception_only(error_type, value))
         s.append("\n")
 
         if self.logger is not None:
@@ -93,7 +99,7 @@ class Debugger(BaseComponent):
         """
 
         try:
-            if not self.events:
+            if not self._events:
                 return
 
             channels = event.channels
@@ -107,7 +113,10 @@ class Debugger(BaseComponent):
             s = repr(event)
 
             if self.prefix:
-                s = "%s: %s" % (self.prefix, s)
+                if hasattr(self.prefix, '__call__'):
+                    s = "%s: %s" % (self.prefix(), s)
+                else:
+                    s = "%s: %s" % (self.prefix, s)
 
             if self.trim:
                 s = "%s ...>" % s[:self.trim]
@@ -115,12 +124,9 @@ class Debugger(BaseComponent):
             if self.logger is not None:
                 self.logger.debug(s)
             else:
-                try:
-                    self.file.write(s)
-                    self.file.write("\n")
-                    self.file.flush()
-                except IOError:
-                    pass
+                self.file.write(s)
+                self.file.write("\n")
+                self.file.flush()
         except Exception as e:
-            print("ERROR (Debugger): {}".format(e))
-            print(format_exc())
+            sys.stderr.write("ERROR (Debugger): {}".format(e))
+            sys.stderr.write("{}".format(format_exc()))
