@@ -1,14 +1,10 @@
-# Package:  handlers
-# Date:     11th April 2010
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """
 This module define the @handler decorator/function and the HandlesType type.
 """
-
-from inspect import getargspec
 from collections import Callable
 
+from circuits.tools import getargspec
+
 
 def handler(*names, **kwargs):
     """Creates an Event Handler
@@ -33,13 +29,6 @@ def handler(*names, **kwargs):
     for a specific event are invoked. The higher the priority, the earlier
     the handler is executed.
 
-    A handler may also be specified as a filter by adding
-    the keyword argument ``filter=True`` to the decorator.
-    If such a handler returns a value different from ``None``, no more
-    handlers are invoked for the handled event. Filtering handlers are
-    invoked before normal handlers with the same priority (but after any
-    handlers with higher priority).
-
     If you want to override a handler defined in a base class of your
     component, you must specify ``override=True``, else your method becomes
     an additional handler for the event.
@@ -74,7 +63,7 @@ def handler(*names, **kwargs):
     """
 
     def wrapper(f):
-        if names and type(names[0]) is bool and not names[0]:
+        if names and isinstance(names[0], bool) and not names[0]:
             f.handler = False
             return f
 
@@ -82,7 +71,6 @@ def handler(*names, **kwargs):
 
         f.names = names
         f.priority = kwargs.get("priority", 0)
-        f.filter = kwargs.get("filter", False)
         f.channel = kwargs.get("channel", None)
         f.override = kwargs.get("override", False)
 
@@ -98,18 +86,22 @@ def handler(*names, **kwargs):
 
 
 class Unknown(object):
+
     """Unknown Dummy Component"""
 
 
 def reprhandler(handler):
-    format = "<%s[%s.%s] (%s.%s)>"
+    format = "<handler[%s][%s]%s (%s.%s)>"
+
+    channel = getattr(handler, "channel", "*")
+    if channel is None:
+        channel = "*"
 
-    channel = handler.channel if handler.channel is not None else "*"
     from circuits.core.manager import Manager
     if isinstance(channel, Manager):
         channel = "<instance of " + channel.__class__.__name__ + ">"
-    names = ".".join(handler.names)
-    type = "filter" if handler.filter else "listener"
+
+    names = ",".join(handler.names)
 
     instance = getattr(
         handler, "im_self", getattr(
@@ -119,7 +111,9 @@ def reprhandler(handler):
 
     method = handler.__name__
 
-    return format % (type, channel, names, instance, method)
+    priority = "[%0.2f]" % (handler.priority,) if handler.priority else ""
+
+    return format % (channel, names, priority, instance, method)
 
 
 class HandlerMetaClass(type):
@@ -130,4 +124,7 @@ class HandlerMetaClass(type):
         callables = (x for x in ns.items() if isinstance(x[1], Callable))
         for name, callable in callables:
             if not (name.startswith("_") or hasattr(callable, "handler")):
-                setattr(cls, name, handler(name)(callable))
+                try:
+                    setattr(cls, name, handler(name)(callable))
+                except ValueError as e:
+                    raise ValueError('{} - {} {}'.format(str(e), repr(cls), name))
