@@ -1,14 +1,16 @@
 """
 .. codeauthor: mnl
 """
-
+from signal import SIGINT, SIGTERM
+from sys import stderr
 from threading import Event
+from traceback import format_exception_only
 
-from .handlers import handler
-from .components import BaseComponent
-import sys
 from circuits.core.handlers import reprhandler
 
+from .components import BaseComponent
+from .handlers import handler
+
 
 class FallBackGenerator(BaseComponent):
 
@@ -24,7 +26,7 @@ class FallBackGenerator(BaseComponent):
         When the queue is empty a GenerateEvents event is fired, here
         we sleep for as long as possible to avoid using extra cpu cycles.
 
-        A poller would override with with a higher priority handler.
+        A poller would override this with a higher priority handler.
         e.g: ``@handler("generate_events", priority=0)``
         and provide a different way to idle when the queue is empty.
         """
@@ -66,16 +68,17 @@ class FallBackGenerator(BaseComponent):
         self._continue.set()
 
 
-class FallBackErrorHandler(BaseComponent):
+class FallBackExceptionHandler(BaseComponent):
+
     """
-    If ther is no handler for error events in the component hierarchy, this
+    If there is no handler for error events in the component hierarchy, this
     component's handler is added automatically. It simply prints
     the error information on stderr.
     """
 
-    @handler("error", channel="*")
-    def _on_error(self, error_type, value, traceback,
-                  handler=None, fevent=None):
+    @handler("exception", channel="*")
+    def _on_exception(self, error_type, value, traceback,
+                      handler=None, fevent=None):
         s = []
 
         if handler is None:
@@ -83,8 +86,27 @@ class FallBackErrorHandler(BaseComponent):
         else:
             handler = reprhandler(handler)
 
-        msg = "ERROR %s (%s) {%s}: %s\n" % (handler, fevent, error_type, value)
+        msg = "ERROR {0:s} ({1:s}) ({2:s}): {3:s}\n".format(
+            handler, repr(fevent), repr(error_type), repr(value)
+        )
+
         s.append(msg)
+        s.append('Traceback (most recent call last):\n')
         s.extend(traceback)
+        s.extend(format_exception_only(error_type, value))
         s.append("\n")
-        sys.stderr.write("".join(s))
+        stderr.write("".join(s))
+
+
+class FallBackSignalHandler(BaseComponent):
+
+    """
+    If there is no handler for signal events in the component hierarchy, this
+    component's handler is added automatically. It simply terminates the
+    system if the signal is SIGINT or SIGTERM.
+    """
+
+    @handler("signal", channel="*")
+    def _on_signal(self, signo, stack):
+        if signo in [SIGINT, SIGTERM]:
+            raise SystemExit(0)
