@@ -1,30 +1,13 @@
-# Package:  values
-# Date:     11th April 2010
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """
 This defines the Value object used by components and events.
 """
-
-
+from ..six import PY2, python_2_unicode_compatible, string_types, text_type
 from .events import Event
-from ..six import string_types
-
-
-class ValueChanged(Event):
-    """Value Changed Event
-
-    This Event is triggered when the return Value of an Event Handler has
-    changed its value.
-    """
-
-    def __init__(self, value):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(ValueChanged, self).__init__(value)
 
 
+@python_2_unicode_compatible
 class Value(object):
+
     """Create a new future Value Object
 
     Creates a new future Value Object which is used by Event Objects and the
@@ -76,7 +59,7 @@ class Value(object):
 
     def __iter__(self):
         return iter(map(lambda v: v.value if isinstance(v, Value) else v,
-                    self.value))
+                        self.value))
 
     def __repr__(self):
         "x.__repr__() <==> repr(x)"
@@ -85,12 +68,13 @@ class Value(object):
         if self.result:
             value = repr(self.value)
 
-        format = "<Value (%s) result: %r errors: %r for %r"
+        format = "<Value (%s) result=%r; errors=%r; for %r>"
         return format % (value, self.result, self.errors, self.event)
 
     def __str__(self):
         "x.__str__() <==> str(x)"
-
+        if PY2:
+            return text_type(self.value).encode('utf-8')
         return str(self.value)
 
     def inform(self, force=False):
@@ -103,10 +87,7 @@ class Value(object):
             if isinstance(notify, string_types):
                 e = Event.create(notify, self)
             else:
-                e = ValueChanged.create(
-                    "{0:s}ValueChanged".format(self.event.__class__.__name__),
-                    self
-                )
+                e = self.event.child("value_changed", self)
 
             self.manager.fire(e, self.manager)
 
