@@ -1,245 +1,98 @@
-# Module:   sockets
-# Date:     04th August 2004
-# Author:   James Mills <prologic@shortcircuit.net.au>
-import select
-
 """Socket Components
 
 This module contains various Socket Components for use with Networking.
 """
-
 import os
+import select
 from collections import defaultdict, deque
-
-from errno import EAGAIN, EALREADY, EBADF
-from errno import ECONNABORTED, EINPROGRESS, EINTR, EISCONN, EMFILE, ENFILE
-from errno import ENOBUFS, ENOMEM, ENOTCONN, EPERM, EPIPE, EINVAL, EWOULDBLOCK
+from errno import (
+    EAGAIN, EALREADY, EBADF, ECONNABORTED, EINPROGRESS, EINTR, EINVAL, EISCONN,
+    EMFILE, ENFILE, ENOBUFS, ENOMEM, ENOTCONN, EPERM, EPIPE, EWOULDBLOCK,
+)
+from socket import (
+    AF_INET, AF_INET6, IPPROTO_IP, IPPROTO_TCP, SO_BROADCAST, SO_REUSEADDR,
+    SOCK_DGRAM, SOCK_STREAM, SOL_SOCKET, TCP_NODELAY, error as SocketError,
+    gaierror, getaddrinfo, getfqdn, gethostbyname, gethostname, socket,
+)
+from time import time
 
 from _socket import socket as SocketType
 
-from socket import gaierror, error
-from socket import getfqdn, gethostbyname, socket, getaddrinfo, gethostname
+from circuits.core import BaseComponent, handler
+from circuits.core.pollers import BasePoller, Poller
+from circuits.core.utils import findcmp
+from circuits.six import binary_type
+
+from .events import (
+    close, closed, connect, connected, disconnect, disconnected, error, read,
+    ready, unreachable, write,
+)
+
+try:
+    from socket import AF_UNIX
+except ImportError:
+    AF_UNIX = None
 
-from socket import AF_INET, AF_INET6, IPPROTO_TCP, SOCK_STREAM, SOCK_DGRAM
-from socket import SOL_SOCKET, SO_BROADCAST, SO_REUSEADDR, TCP_NODELAY
 
 try:
     from ssl import wrap_socket as ssl_socket
-    from ssl import CERT_NONE, PROTOCOL_SSLv23, SSLError, SSL_ERROR_WANT_READ, \
-        SSL_ERROR_WANT_WRITE
+    from ssl import CERT_NONE, PROTOCOL_SSLv23
+    from ssl import SSLError, SSL_ERROR_WANT_WRITE, SSL_ERROR_WANT_READ
+
     HAS_SSL = 1
 except ImportError:
     import warnings
     warnings.warn("No SSL support available.")
     HAS_SSL = 0
+    CERT_NONE = None
+    PROTOCOL_SSLv23 = None
 
 
-from circuits.six import binary_type
-from circuits.core.utils import findcmp
-from circuits.core.pollers import BasePoller, Poller
-from circuits.core import handler, Event, BaseComponent
-
 BUFSIZE = 4096  # 4KB Buffer
 BACKLOG = 5000  # 5K Concurrent Connections
 
-###
-### Event Objects
-###
 
+def do_handshake(sock, on_done=None, on_error=None, extra_args=None):
+    """SSL Async Handshake
 
-class Connect(Event):
-    """Connect Event
+    :param on_done: Function called when handshake is complete
+    :type on_done: :function:
 
-    This Event is sent when a new client connection has arrived on a server.
-    This event is also used for client's to initiate a new connection to
-    a remote host.
-
-    .. note ::
-
-       This event is used for both Client and Server Components.
-
-    :param args:  Client: (host, port) Server: (sock, host, port)
-    :type  args: tuple
-
-    :param kwargs: Client: (ssl)
-    :type  kwargs: dict
+    :param on_error: Function called when handshake errored
+    :type on_error: :function:
     """
 
-    def __init__(self, *args, **kwargs):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
+    extra_args = extra_args or ()
 
-        super(Connect, self).__init__(*args, **kwargs)
+    while True:
+        try:
+            sock.do_handshake()
+            break
+        except SSLError as err:
+            if err.args[0] == SSL_ERROR_WANT_READ:
+                select.select([sock], [], [])
+            elif err.args[0] == SSL_ERROR_WANT_WRITE:
+                select.select([], [sock], [])
+            else:
+                callable(on_error) and on_error(sock, err)
+                return
 
+        yield
 
-class Disconnect(Event):
-    """Disconnect Event
-
-    This Event is sent when a client connection has closed on a server.
-    This event is also used for client's to disconnect from a remote host.
-
-    @note: This event is used for both Client and Server Components.
-
-    :param args:  Client: () Server: (sock)
-    :type  tuple: tuple
-    """
-
-    def __init__(self, *args):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Disconnect, self).__init__(*args)
-
-
-class Connected(Event):
-    """Connected Event
-
-    This Event is sent when a client has successfully connected.
-
-    @note: This event is for Client Components.
-
-    :param host: The hostname connected to.
-    :type  str:  str
-
-    :param port: The port connected to
-    :type  int:  int
-    """
-
-    def __init__(self, host, port):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Connected, self).__init__(host, port)
-
-
-class Disconnected(Event):
-    """Disconnected Event
-
-    This Event is sent when a client has disconnected
-
-    @note: This event is for Client Components.
-    """
-
-    def __init__(self):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Disconnected, self).__init__()
-
-
-class Read(Event):
-    """Read Event
-
-    This Event is sent when a client or server connection has read any data.
-
-    @note: This event is used for both Client and Server Components.
-
-    :param args:  Client: (data) Server: (sock, data)
-    :type  tuple: tuple
-    """
-
-    def __init__(self, *args):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Read, self).__init__(*args)
-
-
-class SocketError(Event):
-    """SocketError Event
-
-    This Event is sent when a client or server connection has an error.
-
-    @note: This event is used for both Client and Server Components.
-
-    :param args:  Client: (error) Server: (sock, error)
-    :type  tuple: tuple
-    """
-
-    def __init__(self, *args):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(SocketError, self).__init__(*args)
-
-
-class Write(Event):
-    """Write Event
-
-    This Event is used to notify a client, client connection or server that
-    we have data to be written.
-
-    @note: This event is never sent, it is used to send data.
-    @note: This event is used for both Client and Server Components.
-
-    :param args:  Client: (data) Server: (sock, data)
-    :type  tuple: tuple
-    """
-
-    def __init__(self, *args):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Write, self).__init__(*args)
-
-
-class Close(Event):
-    """Close Event
-
-    This Event is used to notify a client, client connection or server that
-    we want to close.
-
-    @note: This event is never sent, it is used to close.
-    @note: This event is used for both Client and Server Components.
-
-    :param args:  Client: () Server: (sock)
-    :type  tuple: tuple
-    """
-
-    def __init__(self, *args):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Close, self).__init__(*args)
-
-
-class Ready(Event):
-    """Ready Event
-
-    This Event is used to notify the rest of the system that the underlying
-    Client or Server Component is ready to begin processing connections or
-    incoming/outgoing data. (This is triggered as a direct result of having
-    the capability to support multiple client/server components with a single
-    poller component instance in a system).
-
-    @note: This event is used for both Client and Server Components.
-
-    :param component:  The Client/Server Component that is ready.
-    :type  tuple: Component (Client/Server)
-
-    :param bind:  The (host, port) the server has bound to.
-    :type  tuple: (host, port)
-    """
-
-    def __init__(self, component, bind=None):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        args = (component, bind) if bind is not None else (component,)
-        super(Ready, self).__init__(*args)
-
-
-class Closed(Event):
-    """Closed Event
-
-    This Event is sent when a server has closed its listening socket.
-
-    @note: This event is for Server components.
-    """
-
-    def __init__(self):
-        "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
-
-        super(Closed, self).__init__()
+    callable(on_done) and on_done(sock, *extra_args)
 
 
 class Client(BaseComponent):
 
     channel = "client"
 
-    def __init__(self, bind=None, bufsize=BUFSIZE, channel=channel):
-        super(Client, self).__init__(channel=channel)
+    socket_family = AF_INET
+    socket_type = SOCK_STREAM
+    socket_protocol = IPPROTO_IP
+    socket_options = []
+
+    def __init__(self, bind=None, bufsize=BUFSIZE, channel=channel, **kwargs):
+        super(Client, self).__init__(channel=channel, **kwargs)
 
         if isinstance(bind, SocketType):
             self._bind = bind.getsockname()
@@ -275,26 +128,26 @@ class Client(BaseComponent):
         if self._poller is None:
             if isinstance(component, BasePoller):
                 self._poller = component
-                self.fire(Ready(self))
+                self.fire(ready(self))
             else:
                 if component is not self:
                     return
                 component = findcmp(self.root, BasePoller)
                 if component is not None:
                     self._poller = component
-                    self.fire(Ready(self))
+                    self.fire(ready(self))
                 else:
                     self._poller = Poller().register(self)
-                    self.fire(Ready(self))
+                    self.fire(ready(self))
 
     @handler("stopped", channel="*")
     def _on_stopped(self, component):
-        self.fire(Close())
+        self.fire(close())
 
     @handler("read_value_changed")
     def _on_read_value_changed(self, value):
         if isinstance(value, binary_type):
-            self.fire(Write(value))
+            self.fire(write(value))
 
     @handler("prepare_unregister", channel="*")
     def _on_prepare_unregister(self, event, c):
@@ -313,11 +166,14 @@ class Client(BaseComponent):
 
         try:
             self._sock.shutdown(2)
+        except SocketError:
+            pass
+        try:
             self._sock.close()
-        except error:
+        except SocketError:
             pass
 
-        self.fire(Disconnected())
+        self.fire(disconnected())
 
     @handler("close")
     def close(self):
@@ -328,20 +184,25 @@ class Client(BaseComponent):
 
     def _read(self):
         try:
-            if self.secure and self._ssock:
-                data = self._ssock.read(self._bufsize)
-            else:
-                data = self._sock.recv(self._bufsize)
+            try:
+                if self.secure and self._ssock:
+                    data = self._ssock.read(self._bufsize)
+                else:
+                    data = self._sock.recv(self._bufsize)
+            except SSLError as exc:
+                if exc.errno in (SSL_ERROR_WANT_READ, SSL_ERROR_WANT_WRITE):
+                    return
+                raise
 
             if data:
-                self.fire(Read(data)).notify = True
+                self.fire(read(data)).notify = True
             else:
                 self.close()
-        except error as e:
+        except SocketError as e:
             if e.args[0] == EWOULDBLOCK:
                 return
             else:
-                self.fire(SocketError(e))
+                self.fire(error(e))
                 self._close()
 
     def _write(self, data):
@@ -353,11 +214,11 @@ class Client(BaseComponent):
 
             if nbytes < len(data):
                 self._buffer.appendleft(data[nbytes:])
-        except error as e:
+        except SocketError as e:
             if e.args[0] in (EPIPE, ENOTCONN):
                 self._close()
             else:
-                self.fire(SocketError(e))
+                self.fire(error(e))
 
     @handler("write")
     def write(self, data):
@@ -365,15 +226,15 @@ class Client(BaseComponent):
             self._poller.addWriter(self, self._sock)
         self._buffer.append(data)
 
-    @handler("_disconnect", filter=True)
+    @handler("_disconnect", priority=1)
     def __on_disconnect(self, sock):
         self._close()
 
-    @handler("_read", filter=True)
+    @handler("_read", priority=1)
     def __on_read(self, sock):
         self._read()
 
-    @handler("_write", filter=True)
+    @handler("_write", priority=1)
     def __on_write(self, sock):
         if self._buffer:
             data = self._buffer.popleft()
@@ -385,40 +246,34 @@ class Client(BaseComponent):
             elif self._poller.isWriting(self._sock):
                 self._poller.removeWriter(self._sock)
 
+    def _create_socket(self):
+        sock = socket(self.socket_family, self.socket_type, self.socket_protocol)
 
-def _do_handshake_for_non_blocking(ssock):
-    """
-    This is how to do handshake for an ssl socket with underlying 
-    non-blocking socket (according to the Python doc).
-    """
-    while True:
-        try:
-            ssock.do_handshake()
-            break
-        except SSLError as err:
-            if err.args[0] == SSL_ERROR_WANT_READ:
-                select.select([ssock], [], [])
-            elif err.args[0] == SSL_ERROR_WANT_WRITE:
-                select.select([], [ssock], [])
-            else:
-                raise
+        for option in self.socket_options:
+            sock.setsockopt(*option)
+        sock.setblocking(False)
+        if self._bind is not None:
+            sock.bind(self._bind)
+        return sock
 
 
 class TCPClient(Client):
+
     socket_family = AF_INET
+    socket_type = SOCK_STREAM
+    socket_protocol = IPPROTO_TCP
+    socket_options = [
+        (IPPROTO_TCP, TCP_NODELAY, 1),
+    ]
 
-    def _create_socket(self):
-        sock = socket(self.socket_family, SOCK_STREAM, IPPROTO_TCP)
-        if self._bind is not None:
-            sock.bind(self._bind)
+    def init(self, connect_timeout=5, *args, **kwargs):
+        self.connect_timeout = connect_timeout
 
-        sock.setblocking(False)
-        sock.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
-
-        return sock
-
-    @handler("connect")
+    @handler("connect")  # noqa
     def connect(self, host, port, secure=False, **kwargs):
+        # XXX: C901: This has a high McCacbe complexity score of 10.
+        # TODO: Refactor this!
+
         self.host = host
         self.port = port
         self.secure = secure
@@ -426,36 +281,57 @@ class TCPClient(Client):
         if self.secure:
             self.certfile = kwargs.get("certfile", None)
             self.keyfile = kwargs.get("keyfile", None)
+            self.ca_certs = kwargs.get("ca_certs", None)
 
         try:
             r = self._sock.connect((host, port))
-        except error as e:
+        except SocketError as e:
             if e.args[0] in (EBADF, EINVAL,):
                 self._sock = self._create_socket()
                 r = self._sock.connect_ex((host, port))
             else:
                 r = e.args[0]
 
-            if r in (EISCONN, EWOULDBLOCK, EINPROGRESS, EALREADY):
+            if r not in (EISCONN, EWOULDBLOCK, EINPROGRESS, EALREADY):
+                self.fire(unreachable(host, port, e))
+                self.fire(error(e))
+                self._close()
+                return
+
+        stop_time = time() + self.connect_timeout
+        while time() < stop_time:
+            try:
+                self._sock.getpeername()
                 self._connected = True
-            else:
-                raise
+                break
+            except Exception:
+                yield
 
-        self._connected = True
+        if not self._connected:
+            self.fire(unreachable(host, port))
+            return
 
-        self._poller.addReader(self, self._sock)
+        def on_done(sock):
+            self._poller.addReader(self, sock)
+            self.fire(connected(host, port))
 
         if self.secure:
-            self._ssock = ssl_socket(
-                self._sock, self.keyfile, self.certfile,
+            def on_error(sock, err):
+                self.fire(error(sock, err))
+                self._close()
+
+            self._sock = ssl_socket(
+                self._sock, self.keyfile, self.certfile, ca_certs=self.ca_certs,
                 do_handshake_on_connect=False
             )
-            _do_handshake_for_non_blocking(self._ssock)
-
-        self.fire(Connected(host, port))
+            for _ in do_handshake(self._sock, on_done, on_error):
+                yield
+        else:
+            on_done(self._sock)
 
 
 class TCP6Client(TCPClient):
+
     socket_family = AF_INET6
 
     def parse_bind_parameter(self, bind_parameter):
@@ -464,41 +340,38 @@ class TCP6Client(TCPClient):
 
 class UNIXClient(Client):
 
-    def _create_socket(self):
-        from socket import AF_UNIX
-
-        sock = socket(AF_UNIX, SOCK_STREAM)
-        if self._bind is not None:
-            sock.bind(self._bind)
-
-        sock.setblocking(False)
-
-        return sock
+    socket_family = AF_UNIX
+    socket_type = SOCK_STREAM
+    socket_options = []
 
     @handler("ready")
     def ready(self, component):
         if self._poller is not None and self._connected:
             self._poller.addReader(self, self._sock)
 
-    @handler("connect")
+    @handler("connect")  # noqa
     def connect(self, path, secure=False, **kwargs):
+        # XXX: C901: This has a high McCacbe complexity score of 10.
+        # TODO: Refactor this!
+
         self.path = path
         self.secure = secure
 
         if self.secure:
             self.certfile = kwargs.get("certfile", None)
             self.keyfile = kwargs.get("keyfile", None)
+            self.ca_certs = kwargs.get("ca_certs", None)
 
         try:
             r = self._sock.connect_ex(path)
-        except error as e:
+        except SocketError as e:
             r = e.args[0]
 
         if r:
             if r in (EISCONN, EWOULDBLOCK, EINPROGRESS, EALREADY):
                 self._connected = True
             else:
-                self.fire(SocketError(r))
+                self.fire(error(r))
                 return
 
         self._connected = True
@@ -506,23 +379,32 @@ class UNIXClient(Client):
         self._poller.addReader(self, self._sock)
 
         if self.secure:
+            def on_done(sock):
+                self.fire(connected(gethostname(), path))
+
+            def on_error(sock, err):
+                self.fire(error(err))
+
             self._ssock = ssl_socket(
-                self._sock, self.keyfile, self.certfile,
+                self._sock, self.keyfile, self.certfile, ca_certs=self.ca_certs,
                 do_handshake_on_connect=False
             )
-            _do_handshake_for_non_blocking(self._ssock)
-
-        self.fire(Connected(gethostname(), path))
+            for _ in do_handshake(self._ssock, on_done, on_error):
+                yield
+        else:
+            self.fire(connected(gethostname(), path))
 
 
 class Server(BaseComponent):
 
     channel = "server"
+    socket_protocol = IPPROTO_IP
 
     def __init__(self, bind, secure=False, backlog=BACKLOG,
                  bufsize=BUFSIZE, channel=channel, **kwargs):
         super(Server, self).__init__(channel=channel)
 
+        self.socket_options = self.socket_options[:] + kwargs.get('socket_options', [])
         self._bind = self.parse_bind_parameter(bind)
 
         self._backlog = backlog
@@ -538,14 +420,16 @@ class Server(BaseComponent):
         self._poller = None
         self._buffers = defaultdict(deque)
 
-        self.secure = secure
+        self.__starttls = set()
 
-        if self.secure:
-            self.certfile = kwargs.get("certfile", None)
-            self.keyfile = kwargs.get("keyfile", None)
-            self.cert_reqs = kwargs.get("cert_reqs", CERT_NONE)
-            self.ssl_version = kwargs.get("ssl_version", PROTOCOL_SSLv23)
-            self.ca_certs = kwargs.get("ca_certs", None)
+        self.secure = secure
+        self.certfile = kwargs.get("certfile")
+        self.keyfile = kwargs.get("keyfile", None)
+        self.cert_reqs = kwargs.get("cert_reqs", CERT_NONE)
+        self.ssl_version = kwargs.get("ssl_version", PROTOCOL_SSLv23)
+        self.ca_certs = kwargs.get("ca_certs", None)
+        if self.secure and not self.certfile:
+            raise RuntimeError("certfile must be specified for server-side operations")
 
     def parse_bind_parameter(self, bind_parameter):
         return parse_ipv4_parameter(bind_parameter)
@@ -563,7 +447,7 @@ class Server(BaseComponent):
                     return sockname[0]
                 else:
                     return sockname
-            except error:
+            except SocketError:
                 return None
 
     @property
@@ -573,7 +457,7 @@ class Server(BaseComponent):
                 sockname = self._sock.getsockname()
                 if isinstance(sockname, tuple):
                     return sockname[1]
-            except error:
+            except SocketError:
                 return None
 
     @handler("registered", "started", channel="*")
@@ -582,7 +466,7 @@ class Server(BaseComponent):
             if isinstance(component, BasePoller):
                 self._poller = component
                 self._poller.addReader(self, self._sock)
-                self.fire(Ready(self, (self.host, self.port)))
+                self.fire(ready(self, (self.host, self.port)))
             else:
                 if component is not self:
                     return
@@ -590,26 +474,27 @@ class Server(BaseComponent):
                 if component is not None:
                     self._poller = component
                     self._poller.addReader(self, self._sock)
-                    self.fire(Ready(self, (self.host, self.port)))
+                    self.fire(ready(self, (self.host, self.port)))
                 else:
                     self._poller = Poller().register(self)
                     self._poller.addReader(self, self._sock)
-                    self.fire(Ready(self, (self.host, self.port)))
+                    self.fire(ready(self, (self.host, self.port)))
 
     @handler("stopped", channel="*")
     def _on_stopped(self, component):
-        self.fire(Close())
+        self.fire(close())
 
     @handler("read_value_changed")
     def _on_read_value_changed(self, value):
         if isinstance(value.value, binary_type):
             sock = value.event.args[0]
-            self.fire(Write(sock, value.value))
+            self.fire(write(sock, value.value))
 
     def _close(self, sock):
         if sock is None:
             return
-        if not sock == self._sock and sock not in self._clients:
+
+        if sock != self._sock and sock not in self._clients:
             return
 
         self._poller.discard(sock)
@@ -622,17 +507,23 @@ class Server(BaseComponent):
         else:
             self._sock = None
 
+        if sock in self.__starttls:
+            self.__starttls.remove(sock)
+
         try:
             sock.shutdown(2)
+        except SocketError:
+            pass
+        try:
             sock.close()
-        except error:
+        except SocketError:
             pass
 
-        self.fire(Disconnect(sock))
+        self.fire(disconnect(sock))
 
     @handler("close")
     def close(self, sock=None):
-        closed = sock is None
+        is_closed = sock is None
 
         if sock is None:
             socks = [self._sock]
@@ -646,8 +537,8 @@ class Server(BaseComponent):
             elif sock not in self._closeq:
                 self._closeq.append(sock)
 
-        if closed:
-            self.fire(Closed())
+        if is_closed:
+            self.fire(closed())
 
     def _read(self, sock):
         if sock not in self._clients:
@@ -656,14 +547,14 @@ class Server(BaseComponent):
         try:
             data = sock.recv(self._bufsize)
             if data:
-                self.fire(Read(sock, data)).notify = True
+                self.fire(read(sock, data)).notify = True
             else:
                 self.close(sock)
-        except error as e:
+        except SocketError as e:
             if e.args[0] == EWOULDBLOCK:
                 return
             else:
-                self.fire(SocketError(sock, e))
+                self.fire(error(sock, e))
                 self._close(sock)
 
     def _write(self, sock, data):
@@ -674,9 +565,9 @@ class Server(BaseComponent):
             nbytes = sock.send(data)
             if nbytes < len(data):
                 self._buffers[sock].appendleft(data[nbytes:])
-        except error as e:
+        except SocketError as e:
             if e.args[0] not in (EINTR, EWOULDBLOCK, ENOBUFS):
-                self.fire(SocketError(sock, e))
+                self.fire(error(sock, e))
                 self._close(sock)
             else:
                 self._buffers[sock].appendleft(data)
@@ -690,23 +581,7 @@ class Server(BaseComponent):
     def _accept(self):
         try:
             newsock, host = self._sock.accept()
-            if self.secure and HAS_SSL:
-                newsock = ssl_socket(
-                    newsock,
-                    server_side=True,
-                    keyfile=self.keyfile,
-                    ca_certs=self.ca_certs,
-                    certfile=self.certfile,
-                    cert_reqs=self.cert_reqs,
-                    ssl_version=self.ssl_version,
-                    do_handshake_on_connect=False
-                )
-                _do_handshake_for_non_blocking(newsock)
-
-        except SSLError as e:
-            raise
-
-        except error as e:
+        except SocketError as e:
             if e.args[0] in (EWOULDBLOCK, EAGAIN):
                 return
             elif e.args[0] == EPERM:
@@ -732,23 +607,66 @@ class Server(BaseComponent):
             else:
                 raise
 
-        newsock.setblocking(False)
-        self._poller.addReader(self, newsock)
-        self._clients.append(newsock)
-        self.fire(Connect(newsock, *host))
+        if self.secure and HAS_SSL:
+            for _ in self._do_handshake(newsock):
+                yield
+        else:
+            self._on_accept_done(newsock)
 
-    @handler("_disconnect", filter=True)
+    def _do_handshake(self, sock, fire_connect_event=True):
+        sslsock = ssl_socket(
+            sock,
+            server_side=True,
+            keyfile=self.keyfile,
+            ca_certs=self.ca_certs,
+            certfile=self.certfile,
+            cert_reqs=self.cert_reqs,
+            ssl_version=self.ssl_version,
+            do_handshake_on_connect=False
+        )
+
+        for _ in do_handshake(sslsock, self._on_accept_done, self._on_handshake_error, (fire_connect_event,)):
+            yield _
+
+    def _on_accept_done(self, sock, fire_connect_event=True):
+        sock.setblocking(False)
+        self._poller.addReader(self, sock)
+        self._clients.append(sock)
+        if fire_connect_event:
+            try:
+                self.fire(connect(sock, *sock.getpeername()))
+            except SocketError as exc:
+                # errno 107 (ENOTCONN): the client already disconnected
+                self._on_handshake_error(sock, exc)
+
+    def _on_handshake_error(self, sock, err):
+        self.fire(error(sock, err))
+        self._close(sock)
+
+    @handler('starttls')
+    def starttls(self, sock):
+        if not HAS_SSL:
+            raise RuntimeError('Cannot start TLS. No TLS support.')
+        if sock in self.__starttls:
+            raise RuntimeError('Cannot reuse socket for already started STARTTLS.')
+        self.__starttls.add(sock)
+        self._poller.removeReader(sock)
+        self._clients.remove(sock)
+        for _ in self._do_handshake(sock, False):
+            yield
+
+    @handler("_disconnect", priority=1)
     def _on_disconnect(self, sock):
         self._close(sock)
 
-    @handler("_read", filter=True)
+    @handler("_read", priority=1)
     def _on_read(self, sock):
         if sock == self._sock:
-            self._accept()
+            return self._accept()
         else:
             self._read(sock)
 
-    @handler("_write", filter=True)
+    @handler("_write", priority=1)
     def _on_write(self, sock):
         if self._buffers[sock]:
             data = self._buffers[sock].popleft()
@@ -761,17 +679,28 @@ class Server(BaseComponent):
             elif self._poller.isWriting(sock):
                 self._poller.removeWriter(sock)
 
+    def _create_socket(self):
+        sock = socket(self.socket_family, self.socket_type, self.socket_protocol)
+
+        for option in self.socket_options:
+            sock.setsockopt(*option)
+        sock.setblocking(False)
+        if self._bind is not None:
+            sock.bind(self._bind)
+        return sock
+
 
 class TCPServer(Server):
+
     socket_family = AF_INET
+    socket_type = SOCK_STREAM
+    socket_options = [
+        (SOL_SOCKET, SO_REUSEADDR, 1),
+        (IPPROTO_TCP, TCP_NODELAY, 1),
+    ]
 
     def _create_socket(self):
-        sock = socket(self.socket_family, SOCK_STREAM)
-
-        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
-        sock.setsockopt(IPPROTO_TCP, TCP_NODELAY, 1)
-        sock.setblocking(False)
-        sock.bind(self._bind)
+        sock = super(TCPServer, self)._create_socket()
         sock.listen(self._backlog)
 
         return sock
@@ -781,12 +710,12 @@ class TCPServer(Server):
 
 
 def parse_ipv4_parameter(bind_parameter):
-    if type(bind_parameter) is int:
+    if isinstance(bind_parameter, int):
         try:
             bind = (gethostbyname(gethostname()), bind_parameter)
         except gaierror:
             bind = ("0.0.0.0", bind_parameter)
-    elif type(bind_parameter) is str and ":" in bind_parameter:
+    elif isinstance(bind_parameter, str) and ":" in bind_parameter:
         host, port = bind_parameter.split(":")
         port = int(port)
         bind = (host, port)
@@ -797,7 +726,7 @@ def parse_ipv4_parameter(bind_parameter):
 
 
 def parse_ipv6_parameter(bind_parameter):
-    if type(bind_parameter) is int:
+    if isinstance(bind_parameter, int):
         try:
             _, _, _, _, bind \
                 = getaddrinfo(getfqdn(), bind_parameter, AF_INET6)[0]
@@ -810,6 +739,7 @@ def parse_ipv6_parameter(bind_parameter):
 
 
 class TCP6Server(TCPServer):
+
     socket_family = AF_INET6
 
     def parse_bind_parameter(self, bind_parameter):
@@ -818,36 +748,30 @@ class TCP6Server(TCPServer):
 
 class UNIXServer(Server):
 
-    def _create_socket(self):
-        from socket import AF_UNIX
+    socket_family = AF_UNIX
+    socket_type = SOCK_STREAM
+    socket_options = [
+        (SOL_SOCKET, SO_REUSEADDR, 1),
+    ]
 
+    def _create_socket(self):
         if os.path.exists(self._bind):
             os.unlink(self._bind)
 
-        sock = socket(AF_UNIX, SOCK_STREAM)
-        sock.bind(self._bind)
-
-        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
-        sock.setblocking(False)
+        sock = super(UNIXServer, self)._create_socket()
         sock.listen(self._backlog)
 
         return sock
 
 
 class UDPServer(Server):
+
     socket_family = AF_INET
-
-    def _create_socket(self):
-        sock = socket(self.socket_family, SOCK_DGRAM)
-
-        sock.bind(self._bind)
-
-        sock.setsockopt(SOL_SOCKET, SO_BROADCAST, 1)
-        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
-
-        sock.setblocking(False)
-
-        return sock
+    socket_type = SOCK_DGRAM
+    socket_options = [
+        (SOL_SOCKET, SO_BROADCAST, 1),
+        (SOL_SOCKET, SO_REUSEADDR, 1)
+    ]
 
     def _close(self, sock):
         self._poller.discard(sock)
@@ -857,18 +781,18 @@ class UDPServer(Server):
 
         try:
             sock.shutdown(2)
-        except error:
+        except SocketError:
             pass
         try:
             sock.close()
-        except error:
+        except SocketError:
             pass
 
-        self.fire(Disconnect(sock))
+        self.fire(disconnect(sock))
 
     @handler("close", override=True)
     def close(self):
-        self.fire(Closed())
+        self.fire(closed())
 
         if self._buffers[self._sock] and self._sock not in self._closeq:
             self._closeq.append(self._sock)
@@ -879,11 +803,11 @@ class UDPServer(Server):
         try:
             data, address = self._sock.recvfrom(self._bufsize)
             if data:
-                self.fire(Read(address, data)).notify = True
-        except error as e:
+                self.fire(read(address, data)).notify = True
+        except SocketError as e:
             if e.args[0] in (EWOULDBLOCK, EAGAIN):
                 return
-            self.fire(SocketError(self._sock, e))
+            self.fire(error(self._sock, e))
             self._close(self._sock)
 
     def _write(self, address, data):
@@ -891,11 +815,11 @@ class UDPServer(Server):
             bytes = self._sock.sendto(data, address)
             if bytes < len(data):
                 self._buffers[self._sock].appendleft(data[bytes:])
-        except error as e:
+        except SocketError as e:
             if e.args[0] in (EPIPE, ENOTCONN):
                 self._close(self._sock)
             else:
-                self.fire(SocketError(self._sock, e))
+                self.fire(error(self._sock, e))
 
     @handler("write", override=True)
     def write(self, address, data):
@@ -907,15 +831,15 @@ class UDPServer(Server):
     def broadcast(self, data, port):
         self.write(("<broadcast>", port), data)
 
-    @handler("_disconnect", filter=True, override=True)
+    @handler("_disconnect", priority=1, override=True)
     def _on_disconnect(self, sock):
         self._close(sock)
 
-    @handler("_read", filter=True, override=True)
+    @handler("_read", priority=1, override=True)
     def _on_read(self, sock):
         self._read()
 
-    @handler("_write", filter=True, override=True)
+    @handler("_write", priority=1, override=True)
     def _on_write(self, sock):
         if self._buffers[self._sock]:
             address, data = self._buffers[self._sock].popleft()
