@@ -1,30 +1,27 @@
-# Module:   errors
-# Date:     11th February 2009
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """Errors
 
 This module implements a set of standard HTTP Errors.
 """
-
-from cgi import escape
-
-try:
-    from urllib.parse import urljoin as _urljoin
-except ImportError:
-    from urlparse import urljoin as _urljoin  # NOQA
+import json
+import traceback
 
 from circuits import Event
 
-from . import utils
 from ..six import string_types
-from .constants import SERVER_URL, SERVER_VERSION
-from .constants import DEFAULT_ERROR_MESSAGE, HTTP_STATUS_CODES
+from .constants import (
+    DEFAULT_ERROR_MESSAGE, HTTP_STATUS_CODES, POWERED_BY, SERVER_URL,
+    SERVER_VERSION,
+)
+
+try:
+    from html import escape
+except ImportError:
+    from cgi import escape  # Deprecated since version 3.2
 
 
-class HTTPError(Event):
-    """An event for signaling an HTTP error
-    """
+class httperror(Event):
+
+    """An event for signaling an HTTP error"""
 
     code = 500
     description = ""
@@ -34,7 +31,7 @@ class HTTPError(Event):
         The constructor creates a new instance and modifies the *response*
         argument to reflect the error.
         """
-        super(HTTPError, self).__init__(request, response, code, **kwargs)
+        super(httperror, self).__init__(request, response, code, **kwargs)
 
         # Override HTTPError subclasses
         self.name = "httperror"
@@ -52,31 +49,49 @@ class HTTPError(Event):
         )
 
         if self.error is not None:
+            stack = self.error[2] if isinstance(self.error[2], (list, tuple)) else traceback.format_tb(self.error[2])
             self.traceback = "ERROR: (%s) %s\n%s" % (
-                self.error[0], self.error[1], "".join(self.error[2])
+                self.error[0], self.error[1], "".join(stack)
             )
         else:
             self.traceback = ""
 
         self.response.close = True
-        self.response.code = self.code
+        self.response.status = self.code
+
+        powered_by = POWERED_BY % ({
+            "url": SERVER_URL,
+            "version": SERVER_VERSION
+        }) if getattr(request.server, 'display_banner', False) else ""
 
         self.data = {
             "code": self.code,
             "name": HTTP_STATUS_CODES.get(self.code, "???"),
             "description": self.description,
             "traceback": escape(self.traceback),
-            "url": SERVER_URL,
-            "version": SERVER_VERSION
+            "powered_by": powered_by
         }
 
     def sanitize(self):
-        if self.code < 300 or self.code > 399:
+        if self.code != 201 and not (299 < self.code < 400):
             if "Location" in self.response.headers:
                 del self.response.headers["Location"]
 
     def __str__(self):
         self.sanitize()
+
+        if self.code < 200 or self.code in (204, 205, 304):
+            return ''
+
+        if "json" in self.response.headers.get("Content-Type", ""):
+            index = ["code", "name", "description"]
+            if self.request.print_debug:
+                index.append("traceback")
+            return json.dumps(dict((key, self.data[key]) for key in index))
+
+        if not self.request.print_debug:
+            self.data["traceback"] = ''
+
         return DEFAULT_ERROR_MESSAGE % self.data
 
     def __repr__(self):
@@ -87,27 +102,30 @@ class HTTPError(Event):
         )
 
 
-class Forbidden(HTTPError):
-    """An event for signaling the HTTP Forbidden error
-    """
+class forbidden(httperror):
+
+    """An event for signaling the HTTP Forbidden error"""
+
     code = 403
 
 
-class Unauthorized(HTTPError):
-    """An event for signaling the HTTP Unauthorized error
-    """
+class unauthorized(httperror):
+
+    """An event for signaling the HTTP Unauthorized error"""
+
     code = 401
 
 
-class NotFound(HTTPError):
-    """An event for signaling the HTTP Not Fouond error
-    """
+class notfound(httperror):
+
+    """An event for signaling the HTTP Not Fouond error"""
+
     code = 404
 
 
-class Redirect(HTTPError):
-    """An event for signaling the HTTP Redirect response
-    """
+class redirect(httperror):
+
+    """An event for signaling the HTTP Redirect response"""
 
     def __init__(self, request, response, urls, code=None):
         """
@@ -115,6 +133,7 @@ class Redirect(HTTPError):
         *response* argument to reflect a redirect response to the
         given *url*.
         """
+
         if isinstance(urls, string_types):
             urls = [urls]
 
@@ -125,7 +144,7 @@ class Redirect(HTTPError):
             #  2. a URL relative to root (e.g. "/dummy")
             #  3. a URL relative to the current path
             # Note that any query string in request is discarded.
-            url = _urljoin(utils.url(request), url)
+            url = request.uri.relative(url).unicode()
             abs_urls.append(url)
         self.urls = urls = abs_urls
 
@@ -141,9 +160,9 @@ class Redirect(HTTPError):
             if code < 300 or code > 399:
                 raise ValueError("status code must be between 300 and 399.")
 
-        super(Redirect, self).__init__(request, response, code)
+        super(redirect, self).__init__(request, response, code)
 
-        if code in (300, 301, 302, 303, 307):
+        if code in (300, 301, 302, 303, 307, 308):
             response.headers["Content-Type"] = "text/html"
             # "The ... URI SHOULD be given by the Location field
             # in the response."
@@ -161,6 +180,8 @@ class Redirect(HTTPError):
                          "<a href='%s'>%s</a>."),
                    307: ("This resource has moved temporarily to "
                          "<a href='%s'>%s</a>."),
+                   308: ("This resource has permanently moved to "
+                         "<a href='%s'>%s</a>."),
                    }[code]
             response.body = "<br />\n".join([msg % (u, u) for u in urls])
             # Previous code may have set C-L, so we have to reset it
