@@ -1,46 +1,29 @@
-# Module:   http
-# Date:     13th September 2007
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """Hyper Text Transfer Protocol
 
 This module implements the server side Hyper Text Transfer Protocol
 or commonly known as HTTP.
 """
-
-
 from io import BytesIO
+from socket import socket
 
-try:
-    from urllib.parse import quote
-    from urllib.parse import urlparse, urlunparse
-except ImportError:
-    from urllib import quote  # NOQA
-    from urlparse import urlparse, urlunparse  # NOQA
-
-
-from circuits.six import text_type
+from circuits.core import BaseComponent, Value, handler
 from circuits.net.events import close, write
-from circuits.core import handler, BaseComponent, Value
+from circuits.net.utils import is_ssl_handshake
+from circuits.six import text_type
+from circuits.six.moves.urllib_parse import quote
 
 from . import wrappers
-from .url import parse_url
-from .utils import is_ssl_handshake
-from .exceptions import HTTPException
-from .events import request, response, stream
-from .parsers import HttpParser, BAD_FIRST_LINE
+from .constants import SERVER_PROTOCOL, SERVER_VERSION
 from .errors import httperror, notfound, redirect
-from .exceptions import Redirect as RedirectException
-from .constants import SERVER_VERSION, SERVER_PROTOCOL
+from .events import request, response, stream
+from .exceptions import HTTPException, Redirect as RedirectException
+from .parsers import BAD_FIRST_LINE, HttpParser
+from .url import parse_url
+from .utils import is_unix_socket
 
 MAX_HEADER_FRAGENTS = 20
 HTTP_ENCODING = 'utf-8'
 
-try:
-    unicode
-except NameError:
-    unicode = str
-
 
 class HTTP(BaseComponent):
     """HTTP Protocol Component
@@ -68,15 +51,7 @@ class HTTP(BaseComponent):
         self._server = server
         self._encoding = encoding
 
-        url = "{0:s}://{1:s}{2:s}".format(
-            (server.secure and "https") or "http",
-            server.host or "0.0.0.0",
-            ":{0:d}".format(server.port or 80)
-            if server.port not in (80, 443)
-            else ""
-        )
-        self.uri = parse_url(url)
-
+        self._uri = None
         self._clients = {}
         self._buffers = {}
 
@@ -90,18 +65,37 @@ class HTTP(BaseComponent):
 
     @property
     def scheme(self):
-        if not hasattr(self, "_server"):
-            return
         return "https" if self._server.secure else "http"
 
     @property
     def base(self):
-        if not hasattr(self, "uri"):
-            return
+        if self.uri is None:
+            return ""
         return self.uri.utf8().rstrip(b"/").decode(self._encoding)
 
-    @handler("stream")
+    @property
+    def uri(self):
+        return self._uri
+
+    @handler("ready", priority=1.0)
+    def _on_ready(self, server, bind):
+        if is_unix_socket(server.host):
+            url = server.host
+        else:
+            url = "{0:s}://{1:s}{2:s}".format(
+                (server.secure and "https") or "http",
+                server.host or "0.0.0.0",
+                ":{0:d}".format(server.port or 80)
+                if server.port not in (80, 443)
+                else ""
+            )
+
+        self._uri = parse_url(url)
+
+    @handler("stream")  # noqa
     def _on_stream(self, res, data):
+        sock = res.request.sock
+
         if data is not None:
             if isinstance(data, text_type):
                 data = data.encode(self._encoding)
@@ -115,7 +109,7 @@ class HTTP(BaseComponent):
                 ]
                 data = b"".join(buf)
 
-            self.fire(write(res.request.sock, data))
+            self.fire(write(sock, data))
 
             if res.body and not res.done:
                 try:
@@ -129,13 +123,15 @@ class HTTP(BaseComponent):
             if res.body:
                 res.body.close()
             if res.chunked:
-                self.fire(write(res.request.sock, b"0\r\n\r\n"))
+                self.fire(write(sock, b"0\r\n\r\n"))
             if res.close:
-                self.fire(close(res.request.sock))
-            del self._clients[res.request.sock]
+                self.fire(close(sock))
+            if sock in self._clients:
+                del self._clients[sock]
+
             res.done = True
 
-    @handler("response")
+    @handler("response")  # noqa
     def _on_response(self, res):
         """``Response`` Event Handler
 
@@ -208,7 +204,7 @@ class HTTP(BaseComponent):
         if sock in self._clients:
             del self._clients[sock]
 
-    @handler("read")
+    @handler("read")  # noqa
     def _on_read(self, sock, data):
         """Read Event Handler
 
@@ -220,7 +216,7 @@ class HTTP(BaseComponent):
         if sock in self._buffers:
             parser = self._buffers[sock]
         else:
-            self._buffers[sock] = parser = HttpParser(0, True, self._encoding)
+            self._buffers[sock] = parser = HttpParser(0, True)
 
             # If we receive an SSL handshake at the start of a request
             # and we're not a secure server, then immediately close the
@@ -323,7 +319,7 @@ class HTTP(BaseComponent):
         res.body = str(event)
         self.fire(response(res))
 
-    @handler("request_success")
+    @handler("request_success")  # noqa
     def _on_request_success(self, e, value):
         """
         Handler for the ``RequestSuccess`` event that is automatically
@@ -424,27 +420,35 @@ class HTTP(BaseComponent):
             res.body = value
             self.fire(response(res))
 
-    @handler("error")
-    def _on_error(self, etype, evalue, etraceback, handler=None, fevent=None):
-        if isinstance(fevent, response):
-            res = fevent.args[0]
-            sock = res.request.sock
-        else:
-            sock = fevent.args[0]
-
-        try:
-            req = wrappers.Request(sock, "", "", "", (1, 1), "")
-        except:
-            # If we can't work with the socket, do nothing.
+    @handler("exception")
+    def _on_exception(self, *args, **kwargs):
+        if not len(args) == 3:
             return
 
-        req.server = self._server
+        etype, evalue, etraceback = args
+        fevent = kwargs["fevent"]
 
-        res = wrappers.Response(req, encoding=self._encoding)
+        if isinstance(fevent, response):
+            res = fevent.args[0]
+            req = res.request
+        elif isinstance(fevent.value.parent.event, request):
+            req, res = fevent.value.parent.event.args[:2]
+        elif len(fevent.args[2:]) == 4:
+            req, res = fevent.args[2:]
+        elif len(fevent.args) == 2 and isinstance(fevent.args[0], socket):
+            req = wrappers.Request(fevent.args[0], server=self._server)
+            res = wrappers.Response(req, self._encoding, 500)
+        else:
+            return
+
+        if isinstance(evalue, HTTPException):
+            code = evalue.code
+        else:
+            code = None
 
         self.fire(
             httperror(
-                req, res, error=(etype, evalue, etraceback)
+                req, res, code=code, error=(etype, evalue, etraceback)
             )
         )
 
@@ -489,12 +493,24 @@ class HTTP(BaseComponent):
 
     @handler("request_complete")
     def _on_request_complete(self, *args, **kwargs):
-        """Dummy Event Handler for request_complete"""
+        """Dummy Event Handler for request events
+
+        - request_complete
+        """
 
     @handler("response_success", "response_complete")
     def _on_response_feedback(self, *args, **kwargs):
-        """Dummy Event Handler for response_success and response_complete"""
+        """Dummy Event Handler for response events
+
+        - response_success
+        - response_complete
+        """
 
     @handler("stream_success", "stream_failure", "stream_complete")
     def _on_stream_feedback(self, *args, **kwargs):
-        """Dummy Event Handler for stream_success, stream_failure and stream_complete"""
+        """Dummy Event Handler for stream events
+
+        - stream_success
+        - stream_failure
+        - stream_complete
+        """
