@@ -1,27 +1,21 @@
-# Module:   server
-# Date:     6th November 2008
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """Web Servers
 
 This module implements the several Web Server components.
 """
-
-from circuits.core import handler, BaseComponent
+from sys import stderr
 
 from circuits import io
+from circuits.core import BaseComponent, Timer, handler
+from circuits.net.events import close, read, write
+from circuits.net.sockets import BUFSIZE, TCPServer, UNIXServer
 
-from circuits.net.sockets import Read, Write
-from circuits.net.sockets import TCPServer, UNIXServer
-
-from .http import HTTP
-from .events import WebEvent
-from .wrappers import Request, Host
-from .constants import SERVER_VERSION
 from .dispatchers import Dispatcher
+from .events import terminate
+from .http import HTTP
 
 
 class BaseServer(BaseComponent):
+
     """Create a Base Web Server
 
     Create a Base Web Server (HTTP) bound to the IP Address / Port or
@@ -55,41 +49,29 @@ class BaseServer(BaseComponent):
     channel = "web"
 
     def __init__(self, bind, encoding="utf-8", secure=False, certfile=None,
-                 channel=channel):
+                 channel=channel, display_banner=True, bufsize=BUFSIZE):
         "x.__init__(...) initializes x; see x.__class__.__doc__ for signature"
 
         super(BaseServer, self).__init__(channel=channel)
 
-        if type(bind) in (int, list, tuple):
+        self._display_banner = display_banner
+
+        if isinstance(bind, (int, list, tuple,)):
             SocketType = TCPServer
         else:
-            if ":" in bind:
-                SocketType = TCPServer
-            else:
-                SocketType = UNIXServer
+            SocketType = TCPServer if ":" in bind else UNIXServer
 
         self.server = SocketType(
             bind,
             secure=secure,
             certfile=certfile,
-            channel=channel
+            channel=channel,
+            bufsize=bufsize
         ).register(self)
 
-        HTTP(encoding=encoding, channel=channel).register(self)
-
-        Request.server = self
-        if isinstance(self.server._bind, tuple):
-            Request.local = Host(
-                self.server._bind[0], self.server._bind[1]
-            )
-        else:
-            Request.local = Host(self.server._bind, None)
-        Request.host = self.host
-        Request.scheme = "https" if self.server.secure else "http"
-
-    @property
-    def version(self):
-        return SERVER_VERSION
+        self.http = HTTP(
+            self, encoding=encoding, channel=channel
+        ).register(self)
 
     @property
     def host(self):
@@ -101,33 +83,45 @@ class BaseServer(BaseComponent):
         if hasattr(self, "server"):
             return self.server.port
 
+    @property
+    def display_banner(self):
+        return getattr(self, "_display_banner", False)
+
     @property
     def secure(self):
         if hasattr(self, "server"):
             return self.server.secure
 
-    @property
-    def scheme(self):
-        return "https" if self.secure else "http"
+    @handler("connect")
+    def _on_connect(self, *args, **kwargs):
+        """Dummy Event Handler for connect"""
 
-    @property
-    def base(self):
-        host = self.host or "0.0.0.0"
-        port = self.port or 80
-        scheme = self.scheme
-        secure = self.secure
+    @handler("closed")
+    def _on_closed(self, *args, **kwargs):
+        """Dummy Event Handler for closed"""
 
-        tpl = "%s://%s%s"
+    @handler("signal")
+    def _on_signal(self, *args, **kwargs):
+        """signal Event Handler"""
 
-        if (port == 80 and not secure) or (port == 443 and secure):
-            port = ""
-        else:
-            port = ":%d" % port
+        self.fire(close())
+        Timer(3, terminate()).register(self)
 
-        return tpl % (scheme, host, port)
+    @handler("terminate")
+    def _on_terminate(self):
+        raise SystemExit(0)
+
+    @handler("ready")
+    def _on_ready(self, server, bind):
+        stderr.write(
+            "{0:s} ready! Listening on: {1:s}\n".format(
+                self.http.version, self.http.base
+            )
+        )
 
 
 class Server(BaseServer):
+
     """Create a Web Server
 
     Create a Web Server (HTTP) complete with the default Dispatcher to
@@ -142,10 +136,11 @@ class Server(BaseServer):
 
         super(Server, self).__init__(bind, **kwargs)
 
-        Dispatcher(channel=self.channel).register(self)
+        Dispatcher(channel=self.channel).register(self.http)
 
 
 class FakeSock():
+
     def getpeername(self):
         return (None, None)
 
@@ -157,23 +152,29 @@ class StdinServer(BaseComponent):
     def __init__(self, encoding="utf-8", channel=channel):
         super(StdinServer, self).__init__(channel=channel)
 
-        WebEvent.channels = (channel,)
+        self.server = (io.stdin + io.stdout).register(self)
+        self.http = HTTP(
+            self, encoding=encoding, channel=channel
+        ).register(self)
 
-        self.server = (
-            io.stdin
-            + io.stdout
-            + HTTP(encoding=encoding, channel=channel)
-        )
-
-        self += self.server
-
-        Request.server = self
         Dispatcher(channel=self.channel).register(self)
 
+    @property
+    def host(self):
+        return io.stdin.filename
+
+    @property
+    def port(self):
+        return 0
+
+    @property
+    def secure(self):
+        return False
+
     @handler("read", channel="stdin")
     def read(self, data):
-        self.fire(Read(FakeSock(), data))
+        self.fire(read(FakeSock(), data))
 
     @handler("write")
     def write(self, sock, data):
-        self.fire(Write(data))
+        self.fire(write(data))
