@@ -1,40 +1,36 @@
-# Module:   wrappers
-# Date:     13th September 2007
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """Request/Response Wrappers
 
 This module implements the Request and Response objects.
 """
-
-
-from time import time
-from io import BytesIO
 from functools import partial
+from io import BytesIO
+from time import time
+
+from circuits.net.sockets import BUFSIZE
+from circuits.six import binary_type, text_type
+
+from .constants import HTTP_STATUS_CODES, SERVER_VERSION
+from .errors import httperror
+from .headers import Headers
+from .url import parse_url
 
 try:
     from Cookie import SimpleCookie
 except ImportError:
     from http.cookies import SimpleCookie  # NOQA
 
-from .url import parse_url
-from .headers import Headers
-from ..six import binary_type
-from .errors import httperror
-from circuits.net.sockets import BUFSIZE
-from .constants import HTTP_STATUS_CODES, SERVER_VERSION
+try:
+    from email.utils import formatdate
+    formatdate = partial(formatdate, usegmt=True)
+except ImportError:
+    from rfc822 import formatdate  # NOQA
+
 
 try:
     unicode
 except NameError:
     unicode = str
 
-try:
-    from email.utils import formatdate
-    formatdate = partial(formatdate, usegmt=True)
-except ImportError:
-    from rfc822 import formatdate as HTTPDate  # NOQA
-
 
 def file_generator(input, chunkSize=BUFSIZE):
     chunk = input.read(chunkSize)
@@ -45,6 +41,7 @@ def file_generator(input, chunkSize=BUFSIZE):
 
 
 class Host(object):
+
     """An internet address.
 
     name should be the client's host name. If not available (because no DNS
@@ -87,6 +84,16 @@ class HTTPStatus(object):
             return self._status > other
         return super(HTTPStatus, self).__gt__(other)
 
+    def __le__(self, other):
+        if isinstance(other, int):
+            return self._status <= other
+        return super(HTTPStatus, self).__le__(other)
+
+    def __ge__(self, other):
+        if isinstance(other, int):
+            return self._status >= other
+        return super(HTTPStatus, self).__ge__(other)
+
     def __eq__(self, other):
         if isinstance(other, int):
             return self._status == other
@@ -100,6 +107,9 @@ class HTTPStatus(object):
             self._status, self._reason
         )
 
+    def __format__(self, format_spec):
+        return format(str(self), format_spec)
+
     @property
     def status(self):
         return self._status
@@ -110,6 +120,7 @@ class HTTPStatus(object):
 
 
 class Request(object):
+
     """Creates a new Request object to hold information about a request.
 
     :param sock: The socket object of the request.
@@ -156,6 +167,7 @@ class Request(object):
         self.path = path
         self.protocol = protocol
         self.qs = qs
+        self.print_debug = getattr(server, "display_banner", False)
 
         self.headers = headers or Headers()
         self.server = server
@@ -164,11 +176,12 @@ class Request(object):
 
         if sock is not None:
             name = sock.getpeername()
-            if name is not None:
-                self.remote = Host(*name)
-            else:
-                name = sock.getsockname()
-                self.remote = Host(name, "", name)
+            try:
+                ip, port = name
+                name = None
+            except ValueError:  # AF_UNIX
+                ip, port = None, None
+            self.remote = Host(ip, port, name)
 
         cookie = self.headers.get("Cookie")
         if cookie is not None:
@@ -197,8 +210,11 @@ class Request(object):
         base = "{0:s}://{1:s}{2:s}/".format(
             self.scheme,
             self.host,
-            ":{0:d}".format(self.port) if self.port not in (80, 443) else ""
+            ":{0:d}".format(self.port)
+            if self.port not in (80, 443)
+            else ""
         )
+
         self.base = parse_url(base)
 
         url = "{0:s}{1:s}{2:s}".format(
@@ -215,8 +231,11 @@ class Request(object):
 
 
 class Body(object):
+
     """Response Body"""
 
+    encode_errors = 'strict'
+
     def __get__(self, response, cls=None):
         if response is None:
             return self
@@ -232,6 +251,11 @@ class Body(object):
                 value = [value]
             else:
                 value = []
+        elif isinstance(value, text_type):
+            if value:
+                value = [value.encode(response.encoding, self.encode_errors)]
+            else:
+                value = []
         elif hasattr(value, "read"):
             response.stream = True
             value = file_generator(value)
@@ -244,6 +268,7 @@ class Body(object):
 
 
 class Status(object):
+
     """Response Status"""
 
     def __get__(self, response, cls=None):
@@ -259,6 +284,7 @@ class Status(object):
 
 
 class Response(object):
+
     """Response(sock, request) -> new Response object
 
     A Response object that holds the response to
@@ -288,10 +314,11 @@ class Response(object):
         self.headers = Headers()
         self.headers["Date"] = formatdate()
 
-        if self.request.server is not None:
-            self.headers.add_header("Server", self.request.server.http.version)
-        else:
-            self.headers.add_header("X-Powered-By", SERVER_VERSION)
+        if getattr(self.request.server, "display_banner", False):
+            if self.request.server is not None:
+                self.headers.add_header("Server", request.server.http.version)
+            else:
+                self.headers.add_header("X-Powered-By", SERVER_VERSION)
 
         self.cookie = self.request.cookie
 
@@ -311,7 +338,7 @@ class Response(object):
         return "{0:s} {1:s}\r\n".format(protocol, status)
 
     def __bytes__(self):
-        return str(self).encode(self.encoding)
+        return str(self).encode(self.encoding)  # FIXME: this is wrong. HTTP headers must be ISO8859-1. This should only encode the body as UTF-8.
 
     def prepare(self):
         # Set a default content-Type if we don't have one.
@@ -358,8 +385,7 @@ class Response(object):
                 else:
                     self.close = True
 
-        if (self.request.server is not None
-                and "Connection" not in self.headers):
+        if (self.request.server is not None and "Connection" not in self.headers):
             if self.protocol == "HTTP/1.1":
                 if self.close:
                     self.headers.add_header("Connection", "close")
