@@ -1,19 +1,15 @@
-# Module:   conftest
-# Date:     6th December 2010
-# Author:   James Mills, prologic at shortcircuit dot net dot au
-
 """py.test config"""
 
+import collections
+import sys
+import threading
+from collections import deque
+from time import sleep
+
 import pytest
 
-import sys
-import threading
-import collections
-from time import sleep
-from collections import deque
-
+from circuits import BaseComponent, Debugger, Manager, handler
 from circuits.core.manager import TIMEOUT
-from circuits import handler, BaseComponent, Debugger, Manager
 
 
 class Watcher(BaseComponent):
@@ -27,19 +23,28 @@ class Watcher(BaseComponent):
         with self._lock:
             self.events.append(event)
 
-    def wait(self, name, channel=None, timeout=3.0):
+    def clear(self):
+        self.events.clear()
+
+    def wait(self, name, channel=None, timeout=30.0):
         for i in range(int(timeout / TIMEOUT)):
-            if channel is None:
-                with self._lock:
-                    for event in self.events:
-                        if event.name == name:
-                            return True
-            else:
-                with self._lock:
-                    for event in self.events:
-                        if event.name == name and channel in event.channels:
+            with self._lock:
+                for event in self.events:
+                    if event.name == name and event.waitingHandlers == 0:
+                        if (channel is None) or (channel in event.channels):
                             return True
             sleep(TIMEOUT)
+        else:
+            return False
+
+    def count(self, name, channel=None, n=1, timeout=30.0):
+        n = 0
+        with self._lock:
+            for event in self.events:
+                if event.name == name and event.waitingHandlers == 0:
+                    if (channel is None) or (channel in event.channels):
+                        n += 1
+        return n
 
 
 class Flag(object):
@@ -63,7 +68,7 @@ def call_event(manager, event, *channels):
 
 class WaitEvent(object):
 
-    def __init__(self, manager, name, channel=None, timeout=3.0):
+    def __init__(self, manager, name, channel=None, timeout=30.0):
         if channel is None:
             channel = getattr(manager, "channel", None)
 
@@ -89,7 +94,7 @@ class WaitEvent(object):
             self.manager.removeHandler(self.handler)
 
 
-def wait_for(obj, attr, value=True, timeout=3.0):
+def wait_for(obj, attr, value=True, timeout=30.0):
     from circuits.core.manager import TIMEOUT
     for i in range(int(timeout / TIMEOUT)):
         if isinstance(value, collections.Callable):
@@ -100,7 +105,21 @@ def wait_for(obj, attr, value=True, timeout=3.0):
         sleep(TIMEOUT)
 
 
-@pytest.fixture(scope="session")
+class SimpleManager(Manager):
+
+    def tick(self, timeout=-1):
+        self._running = False
+        return super(SimpleManager, self).tick(timeout)
+
+
+@pytest.fixture
+def simple_manager(request):
+    manager = SimpleManager()
+    Debugger(events=request.config.option.verbose).register(manager)
+    return manager
+
+
+@pytest.fixture
 def manager(request):
     manager = Manager()
 
@@ -113,8 +132,7 @@ def manager(request):
     manager.start()
     assert waiter.wait()
 
-    if request.config.option.verbose:
-        Debugger().register(manager)
+    Debugger(events=request.config.option.verbose).register(manager)
 
     return manager
 
@@ -124,17 +142,21 @@ def watcher(request, manager):
     watcher = Watcher().register(manager)
 
     def finalizer():
+        waiter = WaitEvent(manager, "unregistered")
         watcher.unregister()
+        waiter.wait()
+
+    request.addfinalizer(finalizer)
 
     return watcher
 
 
-def pytest_namespace():
-    return dict((
-        ("WaitEvent", WaitEvent),
-        ("wait_for", wait_for),
-        ("call_event", call_event),
-        ("PLATFORM", sys.platform),
-        ("PYVER", sys.version_info[:3]),
-        ("call_event_from_name", call_event_from_name),
-    ))
+for key, value in dict((
+    ("WaitEvent", WaitEvent),
+    ("wait_for", wait_for),
+    ("call_event", call_event),
+    ("PLATFORM", sys.platform),
+    ("PYVER", sys.version_info[:3]),
+    ("call_event_from_name", call_event_from_name),
+)).items():
+    setattr(pytest, key, value)
